from unittest.mock import AsyncMock, MagicMock, patch

import httpx
import pytest

from brokk_code.executor import ExecutorManager


@pytest.mark.asyncio
async def test_stream_events_polling_logic():
    """
    Verify that stream_events:
    1. Correctstly advances after_seq.
    2. Yields events.
    3. Handles the polling loop and termination state.
    """
    executor = ExecutorManager()
    executor.base_url = "http://127.0.0.1:8080"
    mock_client = AsyncMock(spec=httpx.AsyncClient)
    executor._http_client = mock_client

    job_id = "test-job"

    # Mock responses:
    # 1. Status check -> RUNNING
    # 2. Events check -> returns 2 events, nextAfter 10
    # 3. Status check -> COMPLETED
    # 4-6. Events checks -> empty terminal drain polls, then loop terminates

    status_running = MagicMock(spec=httpx.Response)
    status_running.status_code = 200
    status_running.json.return_value = {"state": "RUNNING"}

    status_completed = MagicMock(spec=httpx.Response)
    status_completed.status_code = 200
    status_completed.json.return_value = {"state": "COMPLETED"}

    events_payload = MagicMock(spec=httpx.Response)
    events_payload.status_code = 200
    events_payload.json.return_value = {
        "events": [
            {"seq": 9, "type": "LLM_TOKEN", "data": {"token": "a"}},
            {"seq": 10, "type": "LLM_TOKEN", "data": {"token": "b"}},
        ],
        "nextAfter": 10,
    }

    empty_events = MagicMock(spec=httpx.Response)
    empty_events.status_code = 200
    empty_events.json.return_value = {"events": [], "nextAfter": 10}

    mock_client.get.side_effect = [
        status_running,  # Loop 1 Status
        events_payload,  # Loop 1 Events
        status_completed,  # Loop 2 Status
        empty_events,  # Loop 2 Events (terminal drain 1)
        empty_events,  # Loop 3 Events (terminal drain 2)
        empty_events,  # Loop 4 Events (terminal drain 3 -> exit)
        status_completed,  # Final status fetch before synthetic yield
    ]

    # Use a small sleep to speed up test
    with (
        patch("asyncio.sleep", AsyncMock()),
        patch("asyncio.get_event_loop") as mock_loop_factory,
    ):
        mock_loop = MagicMock()
        # Advance time by 3.0s on each call to trigger status_interval (2.0s) checks.
        # stream_events calls time() at the start of each while loop iteration.
        # We provide a very long sequence to avoid StopAsyncIteration/RuntimeError
        # if the loop runs many times.
        mock_loop.time.side_effect = [float(i * 3.0) for i in range(1000)]
        mock_loop_factory.return_value = mock_loop
        collected_events = []
        async for event in executor.stream_events(job_id):
            collected_events.append(event)

    assert len(collected_events) == 3
    assert collected_events[0]["seq"] == 9
    assert collected_events[1]["seq"] == 10
    assert collected_events[2] == {"type": "STATE_CHANGE", "data": {"state": "COMPLETED"}}

    # Verify calls
    # Call 1: Status
    # Call 2: Events with after=-1
    # Call 3: Status
    # Call 4+: terminal drain events with after=10
    mock_client.get.assert_any_call(f"/v1/jobs/{job_id}/events?after=-1&limit=100")
    mock_client.get.assert_any_call(f"/v1/jobs/{job_id}/events?after=10&limit=100")


@pytest.mark.asyncio
async def test_stream_events_adaptive_backoff():
    """Verify that current_sleep increases when no events are found."""
    executor = ExecutorManager()
    executor.base_url = "http://127.0.0.1:8080"
    mock_client = AsyncMock(spec=httpx.AsyncClient)
    executor._http_client = mock_client

    # Mock responses for 3 iterations of "no events"
    status_running = MagicMock(spec=httpx.Response)
    status_running.status_code = 200
    status_running.json.return_value = {"state": "RUNNING"}

    status_completed = MagicMock(spec=httpx.Response)
    status_completed.status_code = 200
    status_completed.json.return_value = {"state": "COMPLETED"}

    empty_events = MagicMock(spec=httpx.Response)
    empty_events.status_code = 200
    empty_events.json.return_value = {"events": [], "nextAfter": -1}

    # Sequence:
    # 1. Start: last_status_check is -inf -> triggers status check
    # 2. Iter 1: Get status (RUNNING), Get events (Empty) -> last_status_check set to 0.0,
    #    sleep 0.05, last_status_check reset to 0.0
    # 3. Iter 2: now (3.0) - last_status_check (0.0) > 2.0 -> triggers status check
    # 4. Iter 2: Get status (RUNNING), Get events (Empty) -> sleep 0.1,
    #    last_status_check reset to 0.0
    # 5. Iter 3: now (6.0) - last_status_check (0.0) > 2.0 -> triggers status check
    # 6. Iter 3: Get status (COMPLETED), Get events (Empty) -> terminal drain
    # 7-8. Additional event polls only (Empty) -> Exit after drain budget
    mock_client.get.side_effect = [
        status_running,
        empty_events,  # Iter 1: sleep 0.05
        status_running,
        empty_events,  # Iter 2: sleep 0.1
        status_completed,
        empty_events,  # Iter 3: terminal drain 1
        empty_events,  # terminal drain 2
        empty_events,  # terminal drain 3 -> exit
        status_completed,  # Final status fetch before synthetic yield
    ]

    with (
        patch("asyncio.sleep", AsyncMock()) as mock_sleep,
        patch("asyncio.get_event_loop") as mock_loop_factory,
    ):
        mock_loop = MagicMock()
        # Advance time to ensure status checks trigger and loop can exit.
        # stream_events calls time() at the start of each while loop iteration.
        mock_loop.time.side_effect = [float(i * 3.0) for i in range(1000)]
        mock_loop_factory.return_value = mock_loop

        async for _ in executor.stream_events("job"):
            pass

        sleep_calls = [call.args[0] for call in mock_sleep.call_args_list]
        assert len(sleep_calls) >= 2
        assert sleep_calls[0] == 0.05
        assert sleep_calls[1] == 0.1


@pytest.mark.asyncio
async def test_stream_events_drains_terminal_race_and_yields_late_notification():
    executor = ExecutorManager()
    executor.base_url = "http://127.0.0.1:8080"
    mock_client = AsyncMock(spec=httpx.AsyncClient)
    executor._http_client = mock_client

    status_completed = MagicMock(spec=httpx.Response)
    status_completed.status_code = 200
    status_completed.json.return_value = {"state": "COMPLETED"}

    empty_events = MagicMock(spec=httpx.Response)
    empty_events.status_code = 200
    empty_events.json.return_value = {"events": [], "nextAfter": 10}

    late_events = MagicMock(spec=httpx.Response)
    late_events.status_code = 200
    late_events.json.return_value = {
        "events": [
            {
                "seq": 11,
                "type": "NOTIFICATION",
                "data": {
                    "level": "INFO",
                    "message": "ISSUE_WRITER: issue created id https://github.com/brokkai/brokk/issues/999",
                },
            }
        ],
        "nextAfter": 11,
    }

    # Status says COMPLETED before final notification is visible.
    # stream_events should keep draining and yield the late notification.
    mock_client.get.side_effect = [
        status_completed,
        empty_events,  # terminal drain 1
        late_events,  # final event appears during terminal drain
        status_completed,  # status re-check after late event
        empty_events,  # terminal drain 1 after late event
        empty_events,  # terminal drain 2
        empty_events,  # terminal drain 3 -> exit
        status_completed,  # Final status fetch before synthetic yield
    ]

    with (
        patch("asyncio.sleep", AsyncMock()),
        patch("asyncio.get_event_loop") as mock_loop_factory,
    ):
        mock_loop = MagicMock()
        mock_loop.time.side_effect = [float(i * 3.0) for i in range(1000)]
        mock_loop_factory.return_value = mock_loop

        collected = []
        async for event in executor.stream_events("job"):
            collected.append(event)

    assert collected[0]["seq"] == 11
    assert collected[-1] == {"type": "STATE_CHANGE", "data": {"state": "COMPLETED"}}



import json
import re
import tempfile
import tomllib
from pathlib import Path
from typing import Any

from brokk_code.zed_config import ExistingBrokkCodeEntryError, atomic_write_settings

_IDENTIFIER_RE = re.compile(r"^[A-Za-z_][A-Za-z0-9_]*$")
_SERVER_NAME = "brokk"
_BROKK_MARKER = "# Brokk"
_BROKK_BEGIN_MANAGED = "<!-- BROKK:BEGIN MANAGED SECTION -->"
_BROKK_END_MANAGED = "<!-- BROKK:END MANAGED SECTION -->"
_BROKK_MARKER_RE = re.compile(f"^{_BROKK_MARKER}$", re.MULTILINE)
_BROKK_MANAGED_RE = re.compile(
    f"{re.escape(_BROKK_BEGIN_MANAGED)}.*?{re.escape(_BROKK_END_MANAGED)}", re.DOTALL
)
_BROKK_CODEX_WORKSPACE_SKILL_NAME = "brokk-mcp-workspace"

_BROKK_INSTRUCTIONS_BODY_CLAUDE = f"""{_BROKK_MARKER}
- Use searchSymbols (not Grep) to find class/function/field definitions by name.
- Use scanUsages (not Grep) to find call sites and usages of a known symbol.
- Use getMethodSources (not Read) to retrieve specific method implementations.
- Use getClassSkeletons (not Read) to understand a class's API and structure.
- Use getClassSources (not Read) only when you need the full class implementation.
- Use getFileSummaries or skimFiles (not Read/Glob) for multi-file overviews.
- Use scan to get oriented when starting a new task.
- Use callCodeAgent (not Edit/Write) for all code changes."""

_BROKK_INSTRUCTIONS_BODY_CODEX = f"""{_BROKK_MARKER}
- Use searchSymbols (not Grep) to find class/function/field definitions by name.
- Use scanUsages (not Grep) to find call sites and usages of a known symbol.
- Use getMethodSources (not Read) to retrieve specific method implementations.
- Use getClassSkeletons (not Read) to understand a class's API and structure.
- Use getClassSources (not Read) only when you need the full class implementation.
- Use getFileSummaries or skimFiles (not Read/Glob) for multi-file overviews.
- Use scan to get oriented when starting a new task.
- Use callCodeAgent (not Edit/Write) for all code changes.
- At the start of each Codex session, activate Brokk MCP for the current workspace by
  calling activateWorkspace, then verify with getActiveWorkspace."""

_BROKK_MANAGED_BLOCK_CLAUDE = (
    f"{_BROKK_BEGIN_MANAGED}\n{_BROKK_INSTRUCTIONS_BODY_CLAUDE}\n{_BROKK_END_MANAGED}"
)
_BROKK_MANAGED_BLOCK_CODEX = (
    f"{_BROKK_BEGIN_MANAGED}\n{_BROKK_INSTRUCTIONS_BODY_CODEX}\n{_BROKK_END_MANAGED}"
)

_LEGACY_BLOCKS = [
    # old 3-line generic block
    f"""{_BROKK_MARKER}
- Prefer Brokk MCP tools for syntax-aware search and edits.
- Prefer callCodeAgent for code changes.
- Avoid shell text search when Brokk syntax-aware tools can answer.""",
    # old generic block with activateWorkspace/getActiveWorkspace
    f"""{_BROKK_MARKER}
- Prefer Brokk MCP tools for syntax-aware search and edits.
- Prefer callCodeAgent for code changes.
- Avoid shell text search when Brokk syntax-aware tools can answer.
- At the start of each Codex session, activate Brokk MCP for the current workspace by
  calling activateWorkspace, then verify with getActiveWorkspace.""",
    # tools-specific guidance block from b7efcf...
    _BROKK_INSTRUCTIONS_BODY_CLAUDE,
]
_BROKK_MCP_PERMISSION_ALLOW: list[str] = [
    "Bash(./gradlew:*)",
    # MCP permissions do not support wildcards for tool names.
    # Allow the entire Brokk MCP server by name instead.
    "mcp__brokk",
]


def _ensure_brokk_instructions(path: Path, managed_block: str) -> None:
    """Manages Brokk instructions in a markdown file with delimiters and migration."""
    if not path.exists():
        path.parent.mkdir(parents=True, exist_ok=True)
        path.write_text(managed_block, encoding="utf-8")
        return

    content = path.read_text(encoding="utf-8")

    # Update existing managed block. If delimiters are present but malformed
    # (e.g. END before BEGIN), recover by rewriting a fresh managed block.
    if _BROKK_BEGIN_MANAGED in content and _BROKK_END_MANAGED in content:
        new_content, replacement_count = _BROKK_MANAGED_RE.subn(managed_block, content)
        if replacement_count > 0:
            if new_content != content:
                path.write_text(new_content, encoding="utf-8")
            return

        path.write_text(managed_block, encoding="utf-8")
        return

    # Check for exact legacy matches or empty files
    trimmed = content.strip()
    if not trimmed or any(trimmed == legacy.strip() for legacy in _LEGACY_BLOCKS):
        path.write_text(managed_block, encoding="utf-8")
        return

    # Preserve custom # Brokk content
    if _BROKK_MARKER_RE.search(content):
        return

    # Append managed block
    separator = "\n\n" if not content.endswith("\n\n") else ""
    if content.endswith("\n") and not content.endswith("\n\n"):
        separator = "\n"
    new_content = content + separator + managed_block

    path.write_text(new_content, encoding="utf-8")


def _toml_key(key: str) -> str:
    if _IDENTIFIER_RE.match(key):
        return key
    return json.dumps(key)


def _toml_value(value: Any) -> str:
    if isinstance(value, str):
        return json.dumps(value)
    if isinstance(value, bool):
        return "true" if value else "false"
    if isinstance(value, int) and not isinstance(value, bool):
        return str(value)
    if isinstance(value, float):
        return str(value)
    if isinstance(value, list):
        return f"[{', '.join(_toml_value(item) for item in value)}]"
    if isinstance(value, dict):
        return (
            "{ " + ", ".join(f"{_toml_key(k)} = {_toml_value(v)}" for k, v in value.items()) + " }"
        )
    if value is None:
        return "null"
    raise TypeError(f"Unsupported TOML value type: {type(value)!r}")


def _serialize_toml(data: dict[str, Any]) -> str:
    lines: list[str] = []

    for key, value in data.items():
        if not isinstance(value, dict):
            lines.append(f"{_toml_key(key)} = {_toml_value(value)}")

    for key, value in data.items():
        if not isinstance(value, dict):
            continue

        if key == "mcp_servers":
            for server_name, server_config in value.items():
                lines.append("")
                if not isinstance(server_config, dict):
                    raise ValueError(
                        "Expected every '[mcp_servers.<name>]' entry to be a TOML table"
                    )
                lines.append(f"[{_toml_key(key)}.{_toml_key(server_name)}]")
                for cfg_key, cfg_val in server_config.items():
                    lines.append(f"{_toml_key(cfg_key)} = {_toml_value(cfg_val)}")
            continue

        if value:
            lines.append("")
            lines.append(f"[{_toml_key(key)}]")
            for cfg_key, cfg_val in value.items():
                lines.append(f"{_toml_key(cfg_key)} = {_toml_value(cfg_val)}")
        else:
            lines.append("")
            lines.append(f"[{_toml_key(key)}]")

    return "\n".join(lines) + "\n"


def _atomic_write_toml(path: Path, text: str) -> None:
    with tempfile.NamedTemporaryFile(
        mode="w",
        encoding="utf-8",
        dir=path.parent,
        prefix=f".{path.name}.",
        suffix=".tmp",
        delete=False,
    ) as temp_file:
        temp_file.write(text)
        temp_path = Path(temp_file.name)

    if path.exists():
        temp_path.chmod(path.stat().st_mode)

    temp_path.replace(path)


def _merge_claude_permissions(data: dict[str, Any]) -> None:
    permissions = data.get("permissions")
    if permissions is None:
        permissions = {}
        data["permissions"] = permissions
    elif not isinstance(permissions, dict):
        raise ValueError("Expected 'permissions' to be a JSON object")

    allow_rules = permissions.get("allow")
    if allow_rules is None:
        allow_rules = []
        permissions["allow"] = allow_rules
    elif not isinstance(allow_rules, list):
        raise ValueError("Expected 'permissions.allow' to be an array")

    seen: set[str] = set()
    for existing_rule in allow_rules:
        if not isinstance(existing_rule, str):
            raise ValueError("Expected every entry in 'permissions.allow' to be a string")
        seen.add(existing_rule)

    for rule in _BROKK_MCP_PERMISSION_ALLOW:
        if rule not in seen:
            allow_rules.append(rule)
            seen.add(rule)


def _brokk_mcp_config(uvx_command: str) -> dict[str, Any]:
    return {
        "command": uvx_command,
        "args": ["brokk", "mcp"],
        "type": "stdio",
    }


def configure_claude_code_mcp_settings(
    *, force: bool = False, settings_path: Path | None = None, uvx_command: str = "uvx"
) -> Path:
    """Configure Claude Code MCP settings.

    Args:
        force: Overwrite existing brokk entry if present.
        settings_path: Custom path to .claude.json (default: ~/.claude.json).
        uvx_command: Path to the uvx binary (default: "uvx").
    """
    path = settings_path or Path.home() / ".claude.json"
    if path.exists():
        raw_text = path.read_text(encoding="utf-8")
        try:
            settings = json.loads(raw_text)
        except json.JSONDecodeError as exc:
            raise ValueError(f"Could not parse {path} as JSON: {exc}") from exc
        if not isinstance(settings, dict):
            raise ValueError(f"Expected a JSON object in {path}")
    else:
        settings = {}

    mcp_servers = settings.get("mcpServers")
    if mcp_servers is None:
        mcp_servers = {}
        settings["mcpServers"] = mcp_servers
    elif not isinstance(mcp_servers, dict):
        raise ValueError("Expected 'mcpServers' to be a JSON object")

    if _SERVER_NAME in mcp_servers and not force:
        raise ExistingBrokkCodeEntryError(
            f"mcpServers['{_SERVER_NAME}'] already exists; use --force to overwrite it"
        )

    server_config = _brokk_mcp_config(uvx_command) | {
        "env": {
            "MCP_TIMEOUT": "60000",
            "MCP_TOOL_TIMEOUT": "300000",
        },
    }
    mcp_servers[_SERVER_NAME] = server_config
    _merge_claude_permissions(settings)
    path.parent.mkdir(parents=True, exist_ok=True)
    atomic_write_settings(path, settings)

    # Append to CLAUDE.md
    if settings_path is None:
        claude_md_path = Path.home() / ".claude" / "CLAUDE.md"
    else:
        claude_md_path = path.parent / "CLAUDE.md"

    _ensure_brokk_instructions(claude_md_path, _BROKK_MANAGED_BLOCK_CLAUDE)

    return path


def configure_codex_mcp_settings(
    *, force: bool = False, settings_path: Path | None = None, uvx_command: str = "uvx"
) -> Path:
    """Configure Codex MCP settings.

    Args:
        force: Overwrite existing brokk entry if present.
        settings_path: Custom path to config.toml (default: ~/.codex/config.toml).
        uvx_command: Path to the uvx binary (default: "uvx").
    """
    path = settings_path or Path.home() / ".codex" / "config.toml"
    if path.exists():
        raw_text = path.read_text(encoding="utf-8")
        if raw_text.strip():
            try:
                settings = tomllib.loads(raw_text)
            except ValueError as exc:
                raise ValueError(f"Could not parse {path} as TOML: {exc}") from exc
        else:
            settings = {}
        if not isinstance(settings, dict):
            raise ValueError(f"Expected a TOML object in {path}")
    else:
        settings = {}

    mcp_servers = settings.get("mcp_servers")
    if mcp_servers is None:
        mcp_servers = {}
        settings["mcp_servers"] = mcp_servers
    elif not isinstance(mcp_servers, dict):
        raise ValueError("Expected 'mcp_servers' to be a TOML table")

    if _SERVER_NAME in mcp_servers and not force:
        raise ExistingBrokkCodeEntryError(
            f"mcp_servers['{_SERVER_NAME}'] already exists; use --force to overwrite it"
        )

    server_config = _brokk_mcp_config(uvx_command) | {
        "startup_timeout_sec": 60.0,
        "tool_timeout_sec": 300.0,
    }
    mcp_servers[_SERVER_NAME] = server_config

    path.parent.mkdir(parents=True, exist_ok=True)
    # Add comment explaining uvx usage (TOML supports comments)
    toml_text = "# Brokk uses uvx to always run the latest version\n" + _serialize_toml(settings)
    _atomic_write_toml(path, toml_text)

    # Append to AGENTS.md
    if settings_path is None:
        agents_md_path = Path.home() / ".codex" / "AGENTS.md"
    else:
        agents_md_path = path.parent / "AGENTS.md"

    _ensure_brokk_instructions(agents_md_path, _BROKK_MANAGED_BLOCK_CODEX)

    return path


def _build_codex_workspace_skill_markdown() -> str:
    return f"""---
name: {_BROKK_CODEX_WORKSPACE_SKILL_NAME}
description: Activate Brokk MCP for the current workspace with global Codex MCP config.
---

# Brokk MCP Workspace Activation

Use this skill when Brokk MCP is connected but looking at the wrong repository.

## Steps

1. Determine the current Codex workspace path from this session (do not ask the user).
2. Call Brokk MCP tool `activateWorkspace` with:
   - `workspacePath`: absolute current workspace path
3. Call Brokk MCP tool `getActiveWorkspace` and verify:
   - `activeWorkspacePath` matches the same path (or its normalized git root)
   - `source` is `runtime_override`
"""


def install_codex_mcp_workspace_skill(*, skills_path: Path | None = None) -> Path:
    root = skills_path or (Path.home() / ".codex" / "skills")
    skill_dir = root / _BROKK_CODEX_WORKSPACE_SKILL_NAME
    skill_dir.mkdir(parents=True, exist_ok=True)

    skill_path = skill_dir / "SKILL.md"
    skill_path.write_text(_build_codex_workspace_skill_markdown(), encoding="utf-8")
    return skill_path



from pathlib import Path
from typing import Any

from brokk_code.acp_server import (
    BASE_MODEL_IDS,
    DEFAULT_MODEL_SELECTION,
    DEFAULT_REASONING_LEVEL,
    REASONING_LEVEL_IDS,
    BrokkAcpBridge,
    _available_model_names,
    _build_available_models,
    _extract_session_id_for_cancel,
    _fetch_normalized_catalog_with_retries,
    _known_session_ids,
    _model_variants_for_model,
    _normalize_model_catalog,
    _parse_model_selection,
    _reasoning_options_for_model,
    _sanitize_reasoning_level_for_model,
    acp_slash_commands,
    conversation_payload_to_session_updates,
    extract_prompt_text,
    extract_resource_file_paths,
    get_slash_command,
    map_executor_event_to_session_update,
    normalize_mode,
    resolve_model_selection,
)
from brokk_code.workspace import resolve_workspace_dir


def _text_block(value: str) -> dict[str, str]:
    return {"sessionUpdate": "agent_message_chunk", "text": value}


def _thought_block(value: str) -> dict[str, str]:
    return {"sessionUpdate": "agent_thought_chunk", "text": value}


def test_normalize_mode_defaults_and_known_values() -> None:
    assert normalize_mode(None) == "LUTZ"
    assert normalize_mode("") == "LUTZ"
    assert normalize_mode("ask") == "ASK"
    assert normalize_mode("code") == "CODE"
    assert normalize_mode("lutz") == "LUTZ"
    assert normalize_mode("invalid") == "LUTZ"


def test_model_and_reasoning_constants() -> None:
    assert DEFAULT_MODEL_SELECTION in BASE_MODEL_IDS
    assert DEFAULT_REASONING_LEVEL in REASONING_LEVEL_IDS


def test_resolve_model_selection_variant_and_plain() -> None:
    assert resolve_model_selection(DEFAULT_MODEL_SELECTION) == (DEFAULT_MODEL_SELECTION, None)
    assert resolve_model_selection("gpt-5.3-codex#r=low") == ("gpt-5.3-codex", "low")
    assert resolve_model_selection("gemini-3-flash-preview") == ("gemini-3-flash-preview", None)


def test_normalize_model_catalog_and_reasoning_options() -> None:
    catalog = _normalize_model_catalog(
        {
            "models": [
                {
                    "name": "gpt-5.3-codex",
                    "location": "openai/gpt-5.3-codex",
                    "supportsReasoningEffort": True,
                    "supportsReasoningDisable": True,
                },
                {
                    "name": "gemini-3-flash-preview",
                    "location": "google/gemini-3-flash-preview",
                    "supportsReasoningEffort": False,
                    "supportsReasoningDisable": False,
                },
            ]
        }
    )
    assert [m["name"] for m in catalog] == ["gpt-5.3-codex", "gemini-3-flash-preview"]
    assert _reasoning_options_for_model("gpt-5.3-codex", catalog) == [
        "default",
        "low",
        "medium",
        "high",
        "disable",
    ]
    assert _reasoning_options_for_model("gemini-3-flash-preview", catalog) == [
        "default",
        "low",
        "medium",
        "high",
        "disable",
    ]
    assert (
        _sanitize_reasoning_level_for_model("gemini-3-flash-preview", "high", catalog) == "default"
    )


def test_build_available_models_includes_model_variants_with_conditional_disable() -> None:
    catalog = _normalize_model_catalog(
        {
            "models": [
                {
                    "name": "gpt-5.3-codex",
                    "supportsReasoningEffort": True,
                    "supportsReasoningDisable": True,
                },
                {
                    "name": "gemini-3-flash-preview",
                    "supportsReasoningEffort": False,
                    "supportsReasoningDisable": False,
                },
            ]
        }
    )
    assert _model_variants_for_model("gpt-5.3-codex", catalog) == [
        "low",
        "medium",
        "high",
        "disable",
    ]
    assert _model_variants_for_model("gemini-3-flash-preview", catalog) == []
    options = _build_available_models(catalog)
    assert [value for value, _ in options] == [
        "gpt-5.3-codex",
        "gpt-5.3-codex/low",
        "gpt-5.3-codex/medium",
        "gpt-5.3-codex/high",
        "gpt-5.3-codex/disable",
        "gemini-3-flash-preview",
    ]
    assert [name for _, name in options] == [
        "gpt-5.3-codex",
        "gpt-5.3-codex (low)",
        "gpt-5.3-codex (medium)",
        "gpt-5.3-codex (high)",
        "gpt-5.3-codex (disable)",
        "gemini-3-flash-preview",
    ]
    assert all("#r=" not in value for value, _ in options)


def test_parse_model_selection_routes_model_and_variant() -> None:
    catalog = _normalize_model_catalog(
        {
            "models": [
                {
                    "name": "gpt-5.3-codex",
                    "supportsReasoningEffort": True,
                    "supportsReasoningDisable": True,
                },
            ]
        }
    )
    assert _parse_model_selection("gpt-5.3-codex", catalog) == ("gpt-5.3-codex", None)
    assert _parse_model_selection("gpt-5.3-codex/high", catalog) == ("gpt-5.3-codex", "high")
    assert _parse_model_selection("gpt-5.3-codex/default", catalog) == (
        "gpt-5.3-codex/default",
        None,
    )
    assert _parse_model_selection("model/gpt-5.3-codex", catalog) == ("gpt-5.3-codex", None)


def test_available_model_names_filters_invalid_entries_and_preserves_order() -> None:
    catalog = [
        {"name": "alpha"},
        {"name": None},
        {"name": "   "},
        {"name": "beta"},
        {"name": "alpha"},
        {"location": "missing-name"},
    ]

    assert _available_model_names(catalog) == ["alpha", "beta", "alpha"]


def test_parse_model_selection_ignores_none_model_name() -> None:
    catalog = [{"name": None}, {"name": "gpt-5.3-codex"}]

    assert _parse_model_selection("None", catalog) == ("None", None)
    assert _parse_model_selection("gpt-5.3-codex", catalog) == ("gpt-5.3-codex", None)


async def test_fetch_normalized_catalog_with_retries_recovers_after_transient_failures() -> None:
    calls = {"count": 0}

    async def fetch_payload() -> dict[str, object]:
        calls["count"] += 1
        if calls["count"] < 3:
            raise RuntimeError("executor not ready")
        return {"models": [{"name": "gpt-5.3-codex"}]}

    catalog = await _fetch_normalized_catalog_with_retries(
        fetch_payload,
        attempts=4,
        initial_backoff_seconds=0,
    )

    assert catalog == [
        {
            "name": "gpt-5.3-codex",
            "location": "gpt-5.3-codex",
            "supportsReasoningEffort": False,
            "supportsReasoningDisable": False,
        }
    ]
    assert calls["count"] == 3


async def test_fetch_normalized_catalog_with_retries_returns_none_when_catalog_stays_empty() -> (
    None
):
    calls = {"count": 0}

    async def fetch_payload() -> dict[str, object]:
        calls["count"] += 1
        return {"models": []}

    catalog = await _fetch_normalized_catalog_with_retries(
        fetch_payload,
        attempts=3,
        initial_backoff_seconds=0,
    )

    assert catalog is None
    assert calls["count"] == 3


def test_extract_prompt_text_from_blocks() -> None:
    prompt = [
        {"type": "text", "text": "Hello"},
        {"type": "image", "url": "http://example.com"},
        {"type": "text", "text": "World"},
    ]
    assert extract_prompt_text(prompt) == "Hello\nWorld"


def test_extract_prompt_text_from_string() -> None:
    assert extract_prompt_text("  direct prompt  ") == "direct prompt"


def test_get_slash_command_logic() -> None:
    # Valid commands
    assert get_slash_command("/context") == "/context"
    assert get_slash_command("  /context  ") == "/context"
    assert get_slash_command("/context list files") == "/context"

    # Invalid or non-commands
    assert get_slash_command("context") is None
    assert get_slash_command("/unknown") is None
    assert get_slash_command("Please /context") is None
    assert get_slash_command("") is None


def test_acp_slash_commands_catalog_is_protocol_compatible() -> None:
    commands = acp_slash_commands()
    assert {"name": "context", "description": "Show current context snapshot"} in commands
    assert all(not cmd["name"].startswith("/") for cmd in commands)
    for cmd in commands:
        assert get_slash_command(f"/{cmd['name']}") == f"/{cmd['name']}"


def test_extract_resource_file_paths_supports_zed_resource_shape_and_relative_links() -> None:
    prompt = [
        {
            "type": "resource",
            "resource": {"resource": {"uri": "file:///workspace/src/main.py"}},
        },
        {
            "type": "embedded_resource",
            "resource": {"uri": "file:///workspace/README.md"},
        },
        {"type": "resource_link", "uri": "src/utils.py"},
        {"type": "resource_link", "uri": "https://example.com/docs"},
        {"type": "resource", "resource": {"uri": "zed:///agent/diagnostics"}},
    ]

    assert extract_resource_file_paths(prompt, "/workspace") == [
        "src/main.py",
        "README.md",
        "src/utils.py",
    ]


def test_extract_resource_file_paths_ignores_relative_links_without_cwd() -> None:
    prompt = [{"type": "resource_link", "uri": "src/utils.py"}]
    assert extract_resource_file_paths(prompt, "") == []


def test_map_executor_token_event() -> None:
    event = {"type": "LLM_TOKEN", "data": {"token": "abc"}}
    assert map_executor_event_to_session_update(event, _text_block, _thought_block) == {
        "sessionUpdate": "agent_message_chunk",
        "text": "abc",
    }


def test_map_executor_reasoning_token_event_uses_thought_block() -> None:
    # Boolean True
    event = {"type": "LLM_TOKEN", "data": {"token": "thinking", "isReasoning": True}}
    assert map_executor_event_to_session_update(event, _text_block, _thought_block) == {
        "sessionUpdate": "agent_thought_chunk",
        "text": "thinking",
    }

    # Boolean False
    event = {"type": "LLM_TOKEN", "data": {"token": "not thinking", "isReasoning": False}}
    assert map_executor_event_to_session_update(event, _text_block, _thought_block) == {
        "sessionUpdate": "agent_message_chunk",
        "text": "not thinking",
    }

    # String Truthy
    for val in ["true", "1", "yes", " TRUE "]:
        event = {"type": "LLM_TOKEN", "data": {"token": "thinking", "isReasoning": val}}
        assert map_executor_event_to_session_update(event, _text_block, _thought_block) == {
            "sessionUpdate": "agent_thought_chunk",
            "text": "thinking",
        }, f"Failed for truthy value: {val}"

    # String Falsy (which would be truthy via naive bool())
    for val in ["false", "0", "no", "anything else"]:
        event = {"type": "LLM_TOKEN", "data": {"token": "not thinking", "isReasoning": val}}
        assert map_executor_event_to_session_update(event, _text_block, _thought_block) == {
            "sessionUpdate": "agent_message_chunk",
            "text": "not thinking",
        }, f"Failed for falsy value: {val}"

    # Non-string payloads keep sensible bool behavior.
    for val in [1, [1], {"a": 1}]:
        event = {"type": "LLM_TOKEN", "data": {"token": "thinking", "isReasoning": val}}
        assert map_executor_event_to_session_update(event, _text_block, _thought_block) == {
            "sessionUpdate": "agent_thought_chunk",
            "text": "thinking",
        }, f"Failed for truthy non-string value: {val}"

    for val in [0, None, [], {}]:
        event = {"type": "LLM_TOKEN", "data": {"token": "not thinking", "isReasoning": val}}
        assert map_executor_event_to_session_update(event, _text_block, _thought_block) == {
            "sessionUpdate": "agent_message_chunk",
            "text": "not thinking",
        }, f"Failed for falsy non-string value: {val}"


def test_map_executor_error_event() -> None:
    event = {"type": "ERROR", "data": {"message": "boom"}}
    update = map_executor_event_to_session_update(event, _text_block)
    assert update is not None
    assert "Error:" in update["text"]
    assert "boom" in update["text"]


def test_map_executor_info_notification_event_is_clean() -> None:
    event = {"type": "NOTIFICATION", "data": {"level": "INFO", "message": "planning"}}
    update = map_executor_event_to_session_update(event, _text_block)
    assert update is not None
    assert "planning" in update["text"]
    # Verify no passthrough brackets
    assert "[INFO]" not in update["text"]


def test_map_executor_state_and_cost_events_are_suppressed() -> None:
    state_event = {"type": "STATE_HINT", "data": {"message": "indexing"}}
    assert map_executor_event_to_session_update(state_event, _text_block) is None

    cost_event = {"type": "NOTIFICATION", "data": {"level": "COST", "message": "$0.01"}}
    assert map_executor_event_to_session_update(cost_event, _text_block) is None


def test_map_executor_status_token_mojibake_is_minimally_normalized() -> None:
    token = "reviewingâ€¦"
    event = {"type": "LLM_TOKEN", "data": {"token": token}}
    assert map_executor_event_to_session_update(event, _text_block) == {
        "sessionUpdate": "agent_message_chunk",
        "text": "reviewing...",
    }


def test_map_executor_tool_events_are_suppressed() -> None:
    call_event = {"type": "TOOL_CALL", "data": {"name": "read"}}
    assert map_executor_event_to_session_update(call_event, _text_block) is None

    out_event = {"type": "TOOL_OUTPUT", "data": {"result": "ok"}}
    assert map_executor_event_to_session_update(out_event, _text_block) is None


def test_conversation_payload_to_session_updates_replays_user_assistant_and_reasoning() -> None:
    conversation_data = {
        "entries": [
            {
                "messages": [
                    {"role": "user", "text": "Hello"},
                    {"role": "assistant", "reasoning": "Thinking...", "text": "Hi there"},
                    {"role": "tool", "text": "Tool output"},
                    {"role": "assistant", "text": "   "},
                ]
            },
            {"summary": "Condensed summary"},
        ]
    }

    def _user_update(text: str) -> dict[str, str]:
        return {"sessionUpdate": "user_message_chunk", "text": text}

    def _agent_update(text: str) -> dict[str, str]:
        return {"sessionUpdate": "agent_message_chunk", "text": text}

    def _thought_update(text: str) -> dict[str, str]:
        return {"sessionUpdate": "agent_thought_chunk", "text": text}

    updates = conversation_payload_to_session_updates(
        conversation_data,
        update_user_message_text=_user_update,
        update_agent_message_text=_agent_update,
        update_agent_thought_text=_thought_update,
    )

    assert updates == [
        {"sessionUpdate": "user_message_chunk", "text": "Hello"},
        {"sessionUpdate": "agent_thought_chunk", "text": "Thinking..."},
        {"sessionUpdate": "agent_message_chunk", "text": "Hi there"},
        {"sessionUpdate": "agent_message_chunk", "text": "Tool output"},
        {"sessionUpdate": "agent_message_chunk", "text": "Condensed summary"},
    ]


def test_conversation_payload_to_session_updates_handles_missing_entries() -> None:
    updates = conversation_payload_to_session_updates(
        {"entries": "bad"},
        update_user_message_text=_text_block,
        update_agent_message_text=_thought_block,
    )
    assert updates == []


def test_extract_session_id_for_cancel() -> None:
    assert _extract_session_id_for_cancel((), {"session_id": "abc"}) == "abc"
    assert _extract_session_id_for_cancel(({"sessionId": "def"},), {}) == "def"
    assert _extract_session_id_for_cancel((), {"params": {"sessionId": "ghi"}}) == "ghi"
    assert _extract_session_id_for_cancel((), {}) is None


def test_known_session_ids_collects_ids_without_leaking_names() -> None:
    entries = [
        {"id": "session-a"},
        {"sessionId": "session-b"},
        {"session_id": "session-c"},
        {"id": ""},
        {},
    ]
    assert _known_session_ids(entries) == {"session-a", "session-b", "session-c"}


def test_known_session_ids_handles_bad_payload() -> None:
    assert _known_session_ids(None) == set()
    assert _known_session_ids("bad") == set()


async def test_ensure_ready_bootstraps_session_before_wait_live() -> None:
    calls: list[str] = []

    class StubExecutor:
        def __init__(self) -> None:
            self.workspace_dir = Path("/initial")

        async def start(self) -> None:
            calls.append("start")

    bridge = BrokkAcpBridge(StubExecutor())  # type: ignore[arg-type]
    await bridge.ensure_ready("/tmp/project")

    assert calls == ["start"]
    assert bridge.executor.workspace_dir == resolve_workspace_dir(Path("/tmp/project"))


async def test_start_and_create_session_avoids_bootstrap_on_first_call() -> None:
    calls: list[str] = []

    class StubExecutor:
        def __init__(self) -> None:
            self.workspace_dir = Path("/initial")

        async def start(self) -> None:
            calls.append("start")

        async def create_session(self, name: str = "ignored") -> str:
            calls.append(f"create_session:{name}")
            return "session-real"

        async def wait_live(self) -> bool:
            calls.append("wait_live")
            return True

    bridge = BrokkAcpBridge(StubExecutor())  # type: ignore[arg-type]
    session_id = await bridge.start_and_create_session(name="Requested Session")

    assert session_id == "session-real"
    assert calls == ["start", "create_session:Requested Session", "wait_live"]


async def test_ensure_ready_noops_when_workspace_is_unchanged(tmp_path: Path) -> None:
    calls: list[str] = []

    class StubExecutor:
        def __init__(self, workspace_dir: Path) -> None:
            self.workspace_dir = workspace_dir

        async def start(self) -> None:
            calls.append("start")

    bridge = BrokkAcpBridge(StubExecutor(tmp_path))  # type: ignore[arg-type]

    await bridge.ensure_ready(str(tmp_path))
    await bridge.ensure_ready(str(tmp_path))

    assert calls == ["start"]


async def test_ensure_ready_restarts_executor_when_workspace_changes(tmp_path: Path) -> None:
    calls: list[str] = []
    first = tmp_path / "first"
    second = tmp_path / "second"
    first.mkdir()
    second.mkdir()

    class StubExecutor:
        def __init__(self, workspace_dir: Path) -> None:
            self.workspace_dir = workspace_dir
            self.session_id = "session-1"
            self.base_url = "http://127.0.0.1:9999"

        async def start(self) -> None:
            calls.append(f"start:{self.workspace_dir}")

        async def stop(self) -> None:
            calls.append("stop")

        async def cancel_job(self, job_id: str) -> None:
            calls.append(f"cancel_job:{job_id}")

    bridge = BrokkAcpBridge(StubExecutor(first))  # type: ignore[arg-type]
    bridge._active_job_by_session["acp-1"] = "job-1"

    await bridge.ensure_ready(str(first))
    await bridge.ensure_ready(str(second))

    assert calls == [
        f"start:{first.resolve()}",
        "cancel_job:job-1",
        "stop",
        f"start:{second.resolve()}",
    ]
    assert bridge.executor.workspace_dir == second.resolve()
    assert bridge.executor.session_id is None
    assert bridge.executor.base_url is None


async def test_prompt_standard_flow_calls_submit_job_and_streams_tokens(tmp_path: Path) -> None:
    updates: list[tuple[str, dict[str, str]]] = []
    job_submitted = False

    class StubExecutor:
        def __init__(self, workspace_dir: Path):
            self.workspace_dir = workspace_dir

        async def start(self) -> None:
            pass

        async def create_session(self, name: str = "ignored") -> str:
            return "session-1"

        async def wait_live(self) -> bool:
            return True

        async def switch_session(self, sid: str) -> bool:
            return True

        async def submit_job(
            self,
            task_input: str,
            planner_model: str,
            code_model: str | None = None,
            reasoning_level: str | None = None,
            reasoning_level_code: str | None = None,
            mode: str = "LUTZ",
            session_id: str | None = None,
            **kwargs: Any,
        ) -> str:
            nonlocal job_submitted
            job_submitted = True
            assert task_input == "hello"
            assert session_id == "acp-session-1"
            return "job-1"

        async def stream_events(self, job_id: str):
            yield {"type": "LLM_TOKEN", "data": {"token": "abc"}}

    async def send_update(session_id: str, update: dict[str, str]) -> None:
        updates.append((session_id, update))

    def update_agent_message_text(text: str) -> dict[str, str]:
        return {"sessionUpdate": "agent_message_chunk", "text": text}

    bridge = BrokkAcpBridge(StubExecutor(tmp_path))  # type: ignore[arg-type]
    await bridge.prompt(
        prompt=[{"type": "text", "text": "hello"}],
        session_id="acp-session-1",
        mode="LUTZ",
        planner_model="gpt-5.3-codex",
        code_model="gemini-3-flash-preview",
        reasoning_level="low",
        reasoning_level_code="disable",
        send_update=send_update,
        update_agent_message_text=update_agent_message_text,
    )

    assert job_submitted
    # Only one update for the token "abc"
    assert len(updates) == 1
    assert updates[0][1]["text"] == "abc"


async def test_prompt_context_command_renders_snapshot_without_job(tmp_path: Path) -> None:
    updates: list[tuple[str, dict[str, Any]]] = []
    job_submitted = False

    class StubExecutor:
        async def start(self) -> None:
            pass

        async def create_session(self, name: str) -> str:
            return "session-1"

        async def wait_live(self) -> bool:
            return True

        async def switch_session(self, sid: str) -> bool:
            return True

        async def get_context(self) -> dict[str, Any]:
            return {
                "fragments": [
                    {"shortDescription": "file.py", "tokens": 1500, "pinned": True},
                    {"shortDescription": "other.txt", "tokens": 500, "readonly": True},
                    {"shortDescription": "a.md", "tokens": 400},
                    {"shortDescription": "b.md", "tokens": 300},
                    {"shortDescription": "c.md", "tokens": 200},
                    {"shortDescription": "d.md", "tokens": 100},
                ],
                "usedTokens": 1234,
                "maxTokens": 200000,
                "branch": "main",
                "totalCost": 0.0567,
            }

        async def submit_job(self, **kwargs: Any) -> str:
            nonlocal job_submitted
            job_submitted = True
            return "job-1"

    async def send_update(session_id: str, update: dict[str, Any]) -> None:
        updates.append((session_id, update))

    def update_agent_message_text(text: str) -> dict[str, str]:
        return {"sessionUpdate": "agent_message_chunk", "text": text}

    bridge = BrokkAcpBridge(StubExecutor())  # type: ignore[arg-type]
    # In ACP mode, do NOT emit context snapshots after prompt completion via automatic means.
    # The /context command explicitly generates one.
    await bridge.prompt(
        prompt="/context",
        session_id="acp-1",
        mode="LUTZ",
        planner_model="gpt-5.3-codex",
        code_model=None,
        reasoning_level=None,
        reasoning_level_code=None,
        send_update=send_update,
        update_agent_message_text=update_agent_message_text,
    )

    assert not job_submitted
    assert len(updates) == 1
    assert updates[0][1]["sessionUpdate"] == "agent_message_chunk"
    table = updates[0][1]["text"]
    assert "| Fragment | Tokens | % Context |" in table
    assert "|---|---:|---:|" in table
    assert "| file.py | 1,500 | 0.75% |" in table
    assert "| other.txt | 500 | 0.25% |" in table
    assert "| a.md | 400 | 0.20% |" in table
    assert "| b.md | 300 | 0.15% |" in table
    assert "| (other) | 300 | 0.15% |" in table
    assert "| c.md |" not in table
    assert "| d.md |" not in table
    assert table.index("| file.py |") < table.index("| other.txt |")
    assert "**Total Tokens:** 1,234 / 200,000" in table
    assert "![Token usage](data:image/png;base64," in table



"""Settings modal screens for the Brokk TUI."""

import logging
import shlex
from typing import Any, Dict, List, Optional

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Horizontal, Vertical, VerticalScroll
from textual.screen import ModalScreen
from textual.widgets import (
    Button,
    Checkbox,
    DataTable,
    Input,
    ListItem,
    ListView,
    RadioButton,
    RadioSet,
    Select,
    Static,
    TabbedContent,
    TabPane,
    TextArea,
)

TIMEOUT_OPTIONS: list[tuple[str, str]] = [
    ("No timeout", "-1"),
    ("30", "30"),
    ("60", "60"),
    ("120", "120"),
    ("300", "300"),
    ("600", "600"),
    ("1800", "1800"),
    ("3600", "3600"),
    ("10800", "10800"),
]

# Lazy import to avoid circular dependency
# TaskTitleModalScreen is defined in app.py and used by SettingsModalScreen
# for the exclusion pattern add dialog
_TaskTitleModalScreen = None


def _get_task_title_modal():
    global _TaskTitleModalScreen
    if _TaskTitleModalScreen is None:
        from brokk_code.app import TaskTitleModalScreen

        _TaskTitleModalScreen = TaskTitleModalScreen
    return _TaskTitleModalScreen


logger = logging.getLogger(__name__)


class ModuleEditModalScreen(ModalScreen[Optional[Dict[str, str]]]):
    """Small modal to add or edit a module build entry."""

    BINDINGS = [
        Binding("escape", "dismiss", "Cancel", show=False),
    ]

    def __init__(
        self,
        title: str = "Add Module",
        *,
        alias: str = "",
        language: str = "",
        relative_path: str = "",
        build_lint_command: str = "",
        test_all_command: str = "",
        test_some_command: str = "",
    ) -> None:
        super().__init__()
        self._title = title
        self._alias = alias
        self._language = language
        self._relative_path = relative_path
        self._build_lint_command = build_lint_command
        self._test_all_command = test_all_command
        self._test_some_command = test_some_command

    def compose(self) -> ComposeResult:
        with Vertical(id="module-edit-modal-container"):
            yield Static(self._title, id="module-edit-modal-title")
            yield Static("Alias:", classes="module-edit-label")
            yield Input(value=self._alias, placeholder="e.g., backend", id="module-edit-alias")
            yield Static("Language:", classes="module-edit-label")
            yield Input(value=self._language, placeholder="e.g., Java", id="module-edit-language")
            yield Static("Relative Path:", classes="module-edit-label")
            yield Input(
                value=self._relative_path,
                placeholder="e.g., backend/",
                id="module-edit-path",
            )
            yield Static("Build/Lint Command:", classes="module-edit-label")
            yield Input(
                value=self._build_lint_command,
                placeholder="e.g., ./gradlew build",
                id="module-edit-build-cmd",
            )
            yield Static("Test All Command:", classes="module-edit-label")
            yield Input(
                value=self._test_all_command,
                placeholder="e.g., ./gradlew test",
                id="module-edit-test-all-cmd",
            )
            yield Static("Test Some Command:", classes="module-edit-label")
            yield Input(
                value=self._test_some_command,
                placeholder="e.g., ./gradlew test --tests {{#classes}}",
                id="module-edit-test-some-cmd",
            )
            with Horizontal(id="module-edit-modal-actions"):
                yield Button("Save", id="module-edit-save", variant="primary")
                yield Button("Cancel", id="module-edit-cancel")

    def on_mount(self) -> None:
        self.query_one("#module-edit-alias", Input).focus()

    def on_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id or ""
        if button_id == "module-edit-save":
            self._do_save()
        elif button_id == "module-edit-cancel":
            self.dismiss(None)

    def on_input_submitted(self, event: Input.Submitted) -> None:
        input_id = event.input.id or ""
        field_order = [
            "module-edit-alias",
            "module-edit-language",
            "module-edit-path",
            "module-edit-build-cmd",
            "module-edit-test-all-cmd",
            "module-edit-test-some-cmd",
        ]
        if input_id in field_order:
            idx = field_order.index(input_id)
            if idx < len(field_order) - 1:
                self.query_one(f"#{field_order[idx + 1]}", Input).focus()
            else:
                self._do_save()

    def _do_save(self) -> None:
        result = {
            "alias": self.query_one("#module-edit-alias", Input).value.strip(),
            "language": self.query_one("#module-edit-language", Input).value.strip(),
            "relativePath": self.query_one("#module-edit-path", Input).value.strip(),
            "buildLintCommand": self.query_one("#module-edit-build-cmd", Input).value.strip(),
            "testAllCommand": self.query_one("#module-edit-test-all-cmd", Input).value.strip(),
            "testSomeCommand": self.query_one("#module-edit-test-some-cmd", Input).value.strip(),
        }
        self.dismiss(result)


class SettingsModalScreen(ModalScreen[None]):
    """Full-screen modal for project settings configuration."""

    BINDINGS = [
        Binding("escape", "dismiss", "Cancel", show=False),
    ]

    def __init__(self) -> None:
        super().__init__()
        self._settings_data: Optional[Dict[str, Any]] = None
        self._exclusion_patterns: List[str] = []
        self._modules: List[Dict[str, str]] = []
        self._shell_executable: str = ""
        self._shell_args: str = ""
        self._issue_provider_type: str = "NONE"
        self._github_override: bool = False
        self._analyzer_languages: List[Dict[str, str]] = []
        self._configured_languages: List[str] = []

    def compose(self) -> ComposeResult:
        with Vertical(id="settings-modal-container"):
            yield Static("Settings", id="settings-modal-title")
            with TabbedContent(id="settings-tabs"):
                with TabPane("Build", id="settings-tab-build"):
                    with VerticalScroll(classes="settings-tab-scroll"):
                        # Build Configuration Section
                        yield Static("Build Configuration", classes="settings-section-title")

                        with Horizontal(classes="settings-row"):
                            yield Static("Build/Lint Command:", classes="settings-label")
                            yield Input(
                                placeholder="e.g., make lint", id="settings-build-lint-command"
                            )
                            yield Checkbox("Enabled", id="settings-build-lint-enabled", value=True)

                        with Horizontal(classes="settings-row"):
                            yield Static("Test All Command:", classes="settings-label")
                            yield Input(
                                placeholder="e.g., make test", id="settings-test-all-command"
                            )
                            yield Checkbox("Enabled", id="settings-test-all-enabled", value=True)

                        with Horizontal(classes="settings-row"):
                            yield Static("Test Some Command:", classes="settings-label")
                            yield Input(
                                placeholder="e.g., make test TESTS={testfiles}",
                                id="settings-test-some-command",
                            )
                            yield Checkbox("Enabled", id="settings-test-some-enabled", value=True)

                        with Horizontal(classes="settings-row"):
                            yield Static("Post-Task-List Command:", classes="settings-label")
                            yield Input(
                                placeholder="Command to run after task list completes",
                                id="settings-after-tasklist-command",
                            )

                        with Horizontal(classes="settings-row"):
                            yield Static("Code Agent Test Scope:", classes="settings-label")
                            with RadioSet(id="settings-test-scope"):
                                yield RadioButton(
                                    "Run All Tests", id="settings-scope-all", value=True
                                )
                                yield RadioButton(
                                    "Run Tests in Workspace", id="settings-scope-workspace"
                                )

                        with Horizontal(classes="settings-row"):
                            yield Static("Run Command Timeout (sec):", classes="settings-label")
                            yield Select(
                                TIMEOUT_OPTIONS,
                                id="settings-run-timeout",
                                allow_blank=False,
                                value="-1",
                            )

                        with Horizontal(classes="settings-row"):
                            yield Static("Test Command Timeout (sec):", classes="settings-label")
                            yield Select(
                                TIMEOUT_OPTIONS,
                                id="settings-test-timeout",
                                allow_blank=False,
                                value="-1",
                            )

                        # Modules Section
                        yield Static("Modules", classes="settings-section-title")
                        yield Static(
                            "Configure build commands for submodules/subprojects",
                            classes="settings-hint",
                        )
                        yield DataTable(id="settings-modules-table", cursor_type="row")
                        with Horizontal(classes="settings-list-actions"):
                            yield Button("Add", id="settings-module-add", variant="primary")
                            yield Button("Edit", id="settings-module-edit")
                            yield Button("Remove", id="settings-module-remove")
                            yield Button("\u25b2", id="settings-module-up")
                            yield Button("\u25bc", id="settings-module-down")

                        # Shell Configuration Section
                        yield Static("Shell Configuration", classes="settings-section-title")
                        yield Static(
                            "Configure the shell used to execute build commands",
                            classes="settings-hint",
                        )

                        with Horizontal(classes="settings-row"):
                            yield Static("Execute With:", classes="settings-label")
                            yield Input(
                                placeholder="e.g., /bin/sh",
                                id="settings-shell-executable",
                            )

                        with Horizontal(classes="settings-row"):
                            yield Static("Default Parameters:", classes="settings-label")
                            yield Input(
                                placeholder="e.g., -c",
                                id="settings-shell-args",
                            )

                with TabPane("Code Intelligence", id="settings-tab-ci"):
                    with VerticalScroll(classes="settings-tab-scroll"):
                        yield Static("Analyzer Languages", classes="settings-section-title")
                        yield Static(
                            "Select which languages to analyze for code intelligence",
                            classes="settings-hint",
                        )
                        with VerticalScroll(id="settings-languages-scroll"):
                            pass

                        yield Static("Code Intelligence", classes="settings-section-title")

                        yield Static("Exclusion Patterns:", classes="settings-subsection-label")
                        yield Static(
                            "Patterns to exclude from code intelligence"
                            " (e.g., build, node_modules, *.svg)",
                            classes="settings-hint",
                        )
                        yield ListView(id="settings-exclusion-list")
                        with Horizontal(classes="settings-list-actions"):
                            yield Button("Add", id="settings-exclusion-add", variant="primary")
                            yield Button("Remove", id="settings-exclusion-remove")

                        with Horizontal(classes="settings-row"):
                            yield Static(
                                "Auto-update Local Dependencies:", classes="settings-label"
                            )
                            yield Checkbox("", id="settings-auto-update-local", value=False)

                        with Horizontal(classes="settings-row"):
                            yield Static("Auto-update Git Dependencies:", classes="settings-label")
                            yield Checkbox("", id="settings-auto-update-git", value=False)

                with TabPane("Integrations", id="settings-tab-integrations"):
                    with VerticalScroll(classes="settings-tab-scroll"):
                        # Issue Provider Section
                        yield Static("Issue Provider", classes="settings-section-title")
                        yield Static(
                            "Configure issue tracker integration for fetching issues",
                            classes="settings-hint",
                        )

                        with Horizontal(id="settings-issue-provider-select"):
                            yield Button("None", id="settings-issue-none", variant="primary")
                            yield Button("GitHub", id="settings-issue-github")
                            yield Button("Jira", id="settings-issue-jira")

                        # None provider section
                        with Vertical(id="settings-issue-none-section"):
                            yield Static(
                                "No issue provider configured.",
                                id="settings-issue-none-text",
                            )

                        # GitHub provider section
                        with Vertical(id="settings-issue-github-section", classes="hidden"):
                            yield Checkbox(
                                "Fetch issues from a different GitHub repository",
                                id="settings-github-override",
                                value=False,
                            )
                            with Vertical(id="settings-github-fields", classes="hidden"):
                                with Horizontal(classes="settings-row"):
                                    yield Static("Owner:", classes="settings-label")
                                    yield Input(
                                        placeholder="e.g., owner", id="settings-github-owner"
                                    )
                                with Horizontal(classes="settings-row"):
                                    yield Static("Repository:", classes="settings-label")
                                    yield Input(placeholder="e.g., repo", id="settings-github-repo")
                                with Horizontal(classes="settings-row"):
                                    yield Static("Host (optional):", classes="settings-label")
                                    yield Input(placeholder="github.com", id="settings-github-host")
                            yield Static(
                                "If not overridden, issues are fetched from"
                                " the project's own GitHub repository.",
                                classes="settings-hint",
                            )

                        # Jira provider section
                        with Vertical(id="settings-issue-jira-section", classes="hidden"):
                            with Horizontal(classes="settings-row"):
                                yield Static("Base URL:", classes="settings-label")
                                yield Input(
                                    placeholder="https://yourcompany.atlassian.net",
                                    id="settings-jira-base-url",
                                )
                            with Horizontal(classes="settings-row"):
                                yield Static("API Token:", classes="settings-label")
                                yield Input(
                                    placeholder="API Token",
                                    password=True,
                                    id="settings-jira-api-token",
                                )
                            with Horizontal(classes="settings-row"):
                                yield Static("Project Key:", classes="settings-label")
                                yield Input(
                                    placeholder="e.g., CASSANDRA",
                                    id="settings-jira-project-key",
                                )

                with TabPane("General", id="settings-tab-general"):
                    with VerticalScroll(classes="settings-tab-scroll"):
                        yield Static("Project General", classes="settings-section-title")

                        yield Static("Commit Message Format:", classes="settings-subsection-label")
                        yield TextArea(id="settings-commit-format")

                        yield Static("Data Retention Policy:", classes="settings-subsection-label")
                        with RadioSet(id="settings-data-retention"):
                            yield RadioButton(
                                "Make Brokk Better for Everyone",
                                id="settings-retention-improve",
                                value=True,
                            )
                            yield RadioButton(
                                "Essential Use Only",
                                id="settings-retention-minimal",
                            )
                        yield Static(
                            "Data retention policy affects which AI models are available. "
                            "Deepseek models are not available under Essential Use Only policy.",
                            classes="settings-hint",
                        )

            yield Static("", id="settings-error")

            with Horizontal(id="settings-modal-actions"):
                yield Button("Save", id="settings-save", variant="primary")
                yield Button("Cancel", id="settings-cancel")

    def on_mount(self) -> None:
        self.query_one("#settings-build-lint-command", Input).focus()
        self.app.run_worker(self._load_settings())

    async def _load_settings(self) -> None:
        """Loads current settings from executor and populates form fields."""
        try:
            self._settings_data = await self.app.executor.get_settings()
            self._populate_form()
        except Exception as e:
            error_label = self.query_one("#settings-error", Static)
            error_label.update(f"[bold red]Failed to load settings: {e}[/]")

    def _populate_form(self) -> None:
        """Populates form fields with loaded settings data."""
        if not self._settings_data:
            return

        build_details = self._settings_data.get("buildDetails", {})
        project_settings = self._settings_data.get("projectSettings", {})

        # Build/Lint Command
        self.query_one("#settings-build-lint-command", Input).value = build_details.get(
            "buildLintCommand", ""
        )
        self.query_one("#settings-build-lint-enabled", Checkbox).value = build_details.get(
            "buildLintEnabled", True
        )

        # Test All Command
        self.query_one("#settings-test-all-command", Input).value = build_details.get(
            "testAllCommand", ""
        )
        self.query_one("#settings-test-all-enabled", Checkbox).value = build_details.get(
            "testAllEnabled", True
        )

        # Test Some Command
        self.query_one("#settings-test-some-command", Input).value = build_details.get(
            "testSomeCommand", ""
        )
        self.query_one("#settings-test-some-enabled", Checkbox).value = build_details.get(
            "testSomeEnabled", True
        )

        # Post-Task-List Command
        self.query_one("#settings-after-tasklist-command", Input).value = build_details.get(
            "afterTaskListCommand", ""
        )

        # Code Agent Test Scope
        scope = project_settings.get("codeAgentTestScope", "ALL")
        if scope == "WORKSPACE":
            self.query_one("#settings-scope-workspace", RadioButton).value = True
        else:
            self.query_one("#settings-scope-all", RadioButton).value = True

        # Run Command Timeout
        run_timeout = project_settings.get("runCommandTimeoutSeconds")
        self.query_one("#settings-run-timeout", Select).value = (
            str(run_timeout) if run_timeout is not None else "-1"
        )

        # Test Command Timeout
        test_timeout = project_settings.get("testCommandTimeoutSeconds")
        self.query_one("#settings-test-timeout", Select).value = (
            str(test_timeout) if test_timeout is not None else "-1"
        )

        # Code Intelligence - Exclusion Patterns
        raw_patterns = build_details.get("exclusionPatterns", [])
        self._exclusion_patterns = sorted(set(str(p) for p in raw_patterns if p), key=str.lower)
        self._refresh_exclusion_list()

        # Code Intelligence - Auto-update flags
        self.query_one("#settings-auto-update-local", Checkbox).value = project_settings.get(
            "autoUpdateLocalDependencies", False
        )
        self.query_one("#settings-auto-update-git", Checkbox).value = project_settings.get(
            "autoUpdateGitDependencies", False
        )

        # Modules
        raw_modules = build_details.get("modules", [])
        self._modules = [
            {
                "alias": str(m.get("alias", "")),
                "language": str(m.get("language", "")),
                "relativePath": str(m.get("relativePath", "")),
                "buildLintCommand": str(m.get("buildLintCommand", "")),
                "testAllCommand": str(m.get("testAllCommand", "")),
                "testSomeCommand": str(m.get("testSomeCommand", "")),
            }
            for m in raw_modules
            if isinstance(m, dict)
        ]
        self._refresh_modules_table()

        # Shell Configuration
        shell_config = self._settings_data.get("shellConfig", {})
        self._shell_executable = str(shell_config.get("executable", ""))
        self._shell_args = " ".join(shell_config.get("args", []))
        self.query_one("#settings-shell-executable", Input).value = self._shell_executable
        self.query_one("#settings-shell-args", Input).value = self._shell_args

        # Issue Provider
        issue_provider = self._settings_data.get("issueProvider", {})
        provider_type = str(issue_provider.get("type", "NONE")).upper()
        if provider_type not in ("NONE", "GITHUB", "JIRA"):
            provider_type = "NONE"
        self._set_issue_provider_type(provider_type)

        config = issue_provider.get("config", {})
        if provider_type == "GITHUB":
            owner = str(config.get("owner", ""))
            repo = str(config.get("repo", ""))
            host = str(config.get("host", ""))
            # Check if override is enabled (non-empty fields)
            has_override = bool(owner or repo or host)
            self._github_override = has_override
            self.query_one("#settings-github-override", Checkbox).value = has_override
            self._toggle_github_override(has_override)
            self.query_one("#settings-github-owner", Input).value = owner
            self.query_one("#settings-github-repo", Input).value = repo
            self.query_one("#settings-github-host", Input).value = host
        elif provider_type == "JIRA":
            self.query_one("#settings-jira-base-url", Input).value = str(config.get("baseUrl", ""))
            self.query_one("#settings-jira-api-token", Input).value = str(
                config.get("apiToken", "")
            )
            self.query_one("#settings-jira-project-key", Input).value = str(
                config.get("projectKey", "")
            )

        # Project General - Commit Message Format
        commit_format = project_settings.get("commitMessageFormat", "")
        self.query_one("#settings-commit-format", TextArea).text = commit_format

        # Project General - Data Retention Policy
        data_retention = project_settings.get("dataRetentionPolicy", "IMPROVE_BROKK")
        if data_retention == "MINIMAL":
            self.query_one("#settings-retention-minimal", RadioButton).value = True
        else:
            self.query_one("#settings-retention-improve", RadioButton).value = True

        # Analyzer Languages
        analyzer_langs = self._settings_data.get("analyzerLanguages", {})
        self._configured_languages = analyzer_langs.get("configured", [])
        available = analyzer_langs.get("available", [])
        detected = set(analyzer_langs.get("detected", []))

        # Build display list: union of configured + detected, sorted by name
        configured_set = set(self._configured_languages)
        show_langs = []
        for lang in available:
            internal = lang.get("internalName", "")
            if internal in configured_set or internal in detected:
                show_langs.append(lang)

        # Sort: configured first, then alphabetical within each group
        def _lang_sort_key(x: Dict[str, str]) -> tuple[bool, str]:
            return (x.get("internalName", "") not in configured_set, x.get("name", "").lower())

        show_langs.sort(key=_lang_sort_key)
        self._analyzer_languages = show_langs

        # Populate the scroll container with Checkbox widgets
        lang_scroll = self.query_one("#settings-languages-scroll", VerticalScroll)
        for lang in self._analyzer_languages:
            internal = lang.get("internalName", "")
            name = lang.get("name", internal)
            is_configured = internal in configured_set
            cb = Checkbox(name, value=is_configured, id=f"settings-lang-{internal}")
            lang_scroll.mount(cb)

    def _refresh_exclusion_list(self) -> None:
        """Refreshes the exclusion patterns ListView."""
        list_view = self.query_one("#settings-exclusion-list", ListView)
        list_view.clear()
        for pattern in self._exclusion_patterns:
            list_view.append(ListItem(Static(pattern, markup=False)))

    def _refresh_modules_table(self) -> None:
        """Refreshes the modules DataTable."""
        table = self.query_one("#settings-modules-table", DataTable)
        table.clear(columns=True)
        table.add_columns("Alias", "Language", "Path")
        for module in self._modules:
            table.add_row(
                module.get("alias", ""),
                module.get("language", ""),
                module.get("relativePath", ""),
            )

    def _add_module(self) -> None:
        """Opens a modal to add a new module."""

        def on_result(result: Optional[Dict[str, str]]) -> None:
            if result:
                self._modules.append(result)
                self._refresh_modules_table()

        self.app.push_screen(ModuleEditModalScreen("Add Module"), on_result)

    def _get_selected_module_index(self) -> Optional[int]:
        """Returns the index of the selected row in the modules table, or None."""
        table = self.query_one("#settings-modules-table", DataTable)
        if table.cursor_row is not None and 0 <= table.cursor_row < len(self._modules):
            return table.cursor_row
        return None

    def _edit_selected_module(self) -> None:
        """Opens a modal to edit the selected module."""
        idx = self._get_selected_module_index()
        if idx is None:
            return
        module = self._modules[idx]

        def on_result(result: Optional[Dict[str, str]]) -> None:
            if result:
                self._modules[idx] = result
                self._refresh_modules_table()

        self.app.push_screen(
            ModuleEditModalScreen(
                "Edit Module",
                alias=module.get("alias", ""),
                language=module.get("language", ""),
                relative_path=module.get("relativePath", ""),
                build_lint_command=module.get("buildLintCommand", ""),
                test_all_command=module.get("testAllCommand", ""),
                test_some_command=module.get("testSomeCommand", ""),
            ),
            on_result,
        )

    def _remove_selected_module(self) -> None:
        """Removes the selected module."""
        idx = self._get_selected_module_index()
        if idx is not None:
            del self._modules[idx]
            self._refresh_modules_table()

    def _move_module(self, direction: int) -> None:
        """Moves the selected module up (-1) or down (+1)."""
        idx = self._get_selected_module_index()
        if idx is None:
            return
        new_idx = idx + direction
        if 0 <= new_idx < len(self._modules):
            self._modules[idx], self._modules[new_idx] = (
                self._modules[new_idx],
                self._modules[idx],
            )
            self._refresh_modules_table()
            table = self.query_one("#settings-modules-table", DataTable)
            table.move_cursor(row=new_idx)

    def _set_issue_provider_type(self, provider_type: str) -> None:
        """Sets the issue provider type and shows/hides corresponding sections."""
        self._issue_provider_type = provider_type

        # Update button variants
        none_btn = self.query_one("#settings-issue-none", Button)
        github_btn = self.query_one("#settings-issue-github", Button)
        jira_btn = self.query_one("#settings-issue-jira", Button)

        none_btn.variant = "primary" if provider_type == "NONE" else "default"
        github_btn.variant = "primary" if provider_type == "GITHUB" else "default"
        jira_btn.variant = "primary" if provider_type == "JIRA" else "default"

        # Show/hide sections
        none_section = self.query_one("#settings-issue-none-section")
        github_section = self.query_one("#settings-issue-github-section")
        jira_section = self.query_one("#settings-issue-jira-section")

        if provider_type == "NONE":
            none_section.remove_class("hidden")
            github_section.add_class("hidden")
            jira_section.add_class("hidden")
        elif provider_type == "GITHUB":
            none_section.add_class("hidden")
            github_section.remove_class("hidden")
            jira_section.add_class("hidden")
        elif provider_type == "JIRA":
            none_section.add_class("hidden")
            github_section.add_class("hidden")
            jira_section.remove_class("hidden")

    def _toggle_github_override(self, enabled: bool) -> None:
        """Toggles the GitHub override fields visibility."""
        self._github_override = enabled
        github_fields = self.query_one("#settings-github-fields")
        if enabled:
            github_fields.remove_class("hidden")
        else:
            github_fields.add_class("hidden")

    def on_checkbox_changed(self, event: Checkbox.Changed) -> None:
        """Handle checkbox changes for GitHub override toggle."""
        if event.checkbox.id == "settings-github-override":
            self._toggle_github_override(event.value)

    def on_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id or ""
        if button_id == "settings-save":
            self.app.run_worker(self._save_settings())
        elif button_id == "settings-cancel":
            self.dismiss(None)
        elif button_id == "settings-exclusion-add":
            self._add_exclusion_pattern()
        elif button_id == "settings-exclusion-remove":
            self._remove_selected_exclusion()
        elif button_id == "settings-module-add":
            self._add_module()
        elif button_id == "settings-module-edit":
            self._edit_selected_module()
        elif button_id == "settings-module-remove":
            self._remove_selected_module()
        elif button_id == "settings-module-up":
            self._move_module(-1)
        elif button_id == "settings-module-down":
            self._move_module(1)
        elif button_id == "settings-issue-none":
            self._set_issue_provider_type("NONE")
        elif button_id == "settings-issue-github":
            self._set_issue_provider_type("GITHUB")
        elif button_id == "settings-issue-jira":
            self._set_issue_provider_type("JIRA")

    def on_data_table_row_selected(self, event: DataTable.RowSelected) -> None:
        """Handle Enter key on table rows to edit modules."""
        if event.data_table.id == "settings-modules-table":
            self._edit_selected_module()

    def _add_exclusion_pattern(self) -> None:
        """Opens a modal to add a new exclusion pattern."""

        def on_result(pattern: Optional[str]) -> None:
            if pattern and pattern.strip():
                normalized = pattern.strip()
                if normalized.lower() not in [p.lower() for p in self._exclusion_patterns]:
                    self._exclusion_patterns.append(normalized)
                    self._exclusion_patterns.sort(key=str.lower)
                    self._refresh_exclusion_list()

        self.app.push_screen(
            _get_task_title_modal()("Add Exclusion Pattern", initial=""), on_result
        )

    def _remove_selected_exclusion(self) -> None:
        """Removes the selected exclusion pattern."""
        list_view = self.query_one("#settings-exclusion-list", ListView)
        if list_view.index is not None and 0 <= list_view.index < len(self._exclusion_patterns):
            del self._exclusion_patterns[list_view.index]
            self._refresh_exclusion_list()

    async def _save_settings(self) -> None:
        """Saves form values via executor API."""
        error_label = self.query_one("#settings-error", Static)
        error_label.update("")

        try:
            # Collect build settings (including exclusion patterns and modules)
            build_data: Dict[str, Any] = {
                "buildLintCommand": self.query_one("#settings-build-lint-command", Input).value,
                "buildLintEnabled": self.query_one("#settings-build-lint-enabled", Checkbox).value,
                "testAllCommand": self.query_one("#settings-test-all-command", Input).value,
                "testAllEnabled": self.query_one("#settings-test-all-enabled", Checkbox).value,
                "testSomeCommand": self.query_one("#settings-test-some-command", Input).value,
                "testSomeEnabled": self.query_one("#settings-test-some-enabled", Checkbox).value,
                "afterTaskListCommand": self.query_one(
                    "#settings-after-tasklist-command", Input
                ).value,
                "exclusionPatterns": self._exclusion_patterns,
                "modules": self._modules,
            }

            # Collect project settings
            scope_all = self.query_one("#settings-scope-all", RadioButton).value
            test_scope = "ALL" if scope_all else "WORKSPACE"

            run_timeout_str = str(self.query_one("#settings-run-timeout", Select).value)
            test_timeout_str = str(self.query_one("#settings-test-timeout", Select).value)

            project_data: Dict[str, Any] = {
                "codeAgentTestScope": test_scope,
                "autoUpdateLocalDependencies": self.query_one(
                    "#settings-auto-update-local", Checkbox
                ).value,
                "autoUpdateGitDependencies": self.query_one(
                    "#settings-auto-update-git", Checkbox
                ).value,
                "commitMessageFormat": self.query_one("#settings-commit-format", TextArea).text,
            }

            # Parse timeouts
            if run_timeout_str:
                try:
                    project_data["runCommandTimeoutSeconds"] = int(run_timeout_str)
                except ValueError:
                    error_label.update("[bold red]Run timeout must be a number[/]")
                    return

            if test_timeout_str:
                try:
                    project_data["testCommandTimeoutSeconds"] = int(test_timeout_str)
                except ValueError:
                    error_label.update("[bold red]Test timeout must be a number[/]")
                    return

            # Build shell configuration
            shell_executable = self.query_one("#settings-shell-executable", Input).value.strip()
            shell_args_str = self.query_one("#settings-shell-args", Input).value.strip()
            shell_args = shlex.split(shell_args_str) if shell_args_str else []

            # Build issue provider configuration
            issue_provider_data: Dict[str, Any] = {"type": self._issue_provider_type}
            if self._issue_provider_type == "GITHUB":
                if self._github_override:
                    issue_provider_data["config"] = {
                        "owner": self.query_one("#settings-github-owner", Input).value.strip(),
                        "repo": self.query_one("#settings-github-repo", Input).value.strip(),
                        "host": self.query_one("#settings-github-host", Input).value.strip(),
                    }
                else:
                    issue_provider_data["config"] = {}
            elif self._issue_provider_type == "JIRA":
                issue_provider_data["config"] = {
                    "baseUrl": self.query_one("#settings-jira-base-url", Input).value.strip(),
                    "apiToken": self.query_one("#settings-jira-api-token", Input).value.strip(),
                    "projectKey": self.query_one("#settings-jira-project-key", Input).value.strip(),
                }
            else:
                issue_provider_data["config"] = {}

            # Build data retention policy
            retention_improve = self.query_one("#settings-retention-improve", RadioButton).value
            retention_policy = "IMPROVE_BROKK" if retention_improve else "MINIMAL"

            # Build analyzer languages
            selected_languages = []
            for lang in self._analyzer_languages:
                internal = lang.get("internalName", "")
                try:
                    cb = self.query_one(f"#settings-lang-{internal}", Checkbox)
                    if cb.value:
                        selected_languages.append(internal)
                except Exception:
                    pass

            # Save all settings atomically
            payload: Dict[str, Any] = {
                "buildDetails": build_data,
                "projectSettings": project_data,
                "shellConfig": {"executable": shell_executable, "args": shell_args},
                "issueProvider": issue_provider_data,
                "dataRetentionPolicy": retention_policy,
            }
            if self._analyzer_languages:
                payload["analyzerLanguages"] = {"languages": selected_languages}

            await self.app.executor.update_all_settings(payload)

            # Show success message in chat
            chat = self.app._maybe_chat()
            if chat:
                chat.add_system_message("Settings saved successfully.", level="SUCCESS")

            self.dismiss(None)

        except Exception as e:
            error_label.update(f"[bold red]Failed to save settings: {e}[/]")



"""Modal screen for viewing command history with expandable output."""

from datetime import datetime
from typing import Any, Dict, List, Optional

from textual import events
from textual.binding import Binding
from textual.containers import Horizontal, Vertical, VerticalScroll
from textual.css.query import NoMatches
from textual.screen import ModalScreen
from textual.widgets import ListItem, ListView, Static


class CommandListItem(ListItem):
    """A ListItem that carries associated command data."""

    def __init__(self, *args, cmd_data: Dict[str, Any], **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.cmd_data = cmd_data


class CommandsModalScreen(ModalScreen[None]):
    """Modal showing command history with expandable output."""

    BINDINGS = [
        Binding("q", "dismiss", "Close", show=False),
        Binding("enter", "toggle_output", "Toggle Output", show=False),
    ]

    def __init__(self, command_history: List[Dict[str, Any]]) -> None:
        super().__init__()
        self.command_history = command_history
        self._expanded_id: Optional[str] = None

    def compose(self):
        total = len(self.command_history)
        succeeded = sum(1 for cmd in self.command_history if cmd.get("success", False))
        failed = total - succeeded

        with Vertical(id="commands-modal-container"):
            yield Static("Command History", id="commands-modal-title")

            if not self.command_history:
                with VerticalScroll(id="commands-list-wrap"):
                    yield Static("No commands have been executed yet.", id="commands-empty")
            else:
                with Horizontal(id="commands-header"):
                    yield Static("Time", classes="cmd-col-time")
                    yield Static("Stage", classes="cmd-col-stage")
                    yield Static("Command", classes="cmd-col-command")
                    yield Static("Status", classes="cmd-col-status")

                with VerticalScroll(id="commands-list-wrap"):
                    list_items = []
                    for cmd in self.command_history:
                        ts = cmd.get("timestamp", 0)
                        dt = datetime.fromtimestamp(ts).strftime("%H:%M:%S")
                        stage = cmd.get("stage", "Command")
                        command = cmd.get("command", "")
                        success = cmd.get("success", False)
                        cmd_id = cmd.get("id", "")

                        # Truncate command for display
                        if len(command) > 40:
                            command_display = command[:37] + "..."
                        else:
                            command_display = command

                        status = "[bold green]OK[/]" if success else "[bold red]FAIL[/]"

                        li = CommandListItem(
                            Horizontal(
                                Static(dt, classes="cmd-col-time"),
                                Static(stage, classes="cmd-col-stage"),
                                Static(command_display, classes="cmd-col-command", markup=False),
                                Static(status, classes="cmd-col-status"),
                                classes="command-row",
                            ),
                            cmd_data=cmd,
                            id=f"cmd-{cmd_id}",
                        )
                        list_items.append(li)

                    yield ListView(*list_items, id="commands-list")

                # Output panel for expanded view
                yield Static("", id="commands-output-panel", classes="hidden")

            # Summary
            with Vertical(id="commands-summary"):
                summary_text = f"[bold]{total} command{'s' if total != 1 else ''}[/]"
                if total > 0:
                    summary_text += f" ({succeeded} succeeded, {failed} failed)"
                yield Static(summary_text, id="commands-summary-text")

            yield Static("Enter: Toggle Output  Esc: Close", id="commands-help-line")

    def on_mount(self) -> None:
        if self.query(ListView):
            self.query_one(ListView).focus()
        else:
            self.query_one("#commands-help-line").focus()

    def on_list_view_highlighted(self, message: ListView.Highlighted) -> None:
        """Ensure the highlighted row is scrolled into view and update output if expanded."""
        if message.item:
            try:
                scroll_wrap = self.query_one("#commands-list-wrap")
                scroll_wrap.scroll_to_widget(message.item, animate=False)
            except NoMatches:
                pass
            except Exception:
                pass

            # If we have an expanded view, update it with new selection
            if self._expanded_id is not None:
                self._show_output_for_item(message.item)

    def on_list_view_selected(self, message: ListView.Selected) -> None:
        """Toggle output panel when item is selected."""
        self.action_toggle_output()

    def action_toggle_output(self) -> None:
        """Toggle the output panel for the currently highlighted command."""
        try:
            list_view = self.query_one("#commands-list", ListView)
            output_panel = self.query_one("#commands-output-panel", Static)
        except NoMatches:
            return

        if list_view.highlighted_child is None:
            return

        item = list_view.highlighted_child
        if not isinstance(item, CommandListItem):
            return

        cmd_id = item.cmd_data.get("id", "")

        # Toggle expansion
        if self._expanded_id == cmd_id:
            # Collapse
            self._expanded_id = None
            output_panel.add_class("hidden")
            output_panel.update("")
        else:
            # Expand
            self._expanded_id = cmd_id
            self._show_output_for_item(item)

    def _show_output_for_item(self, item: CommandListItem) -> None:
        """Show the output panel for the given list item."""
        try:
            output_panel = self.query_one("#commands-output-panel", Static)
        except NoMatches:
            return

        if not isinstance(item, CommandListItem):
            return

        self._expanded_id = item.cmd_data.get("id", "")
        output = item.cmd_data.get("output", "").strip()
        exception = item.cmd_data.get("exception")
        command = item.cmd_data.get("command", "")

        parts = [f"[bold]Command:[/] {command}"]
        if output:
            parts.append(f"\n[bold]Output:[/]\n{output}")
        if exception:
            parts.append(f"\n[bold red]Error:[/] {exception}")
        if not output and not exception:
            parts.append("\n[dim]No output[/]")

        output_panel.update("\n".join(parts))
        output_panel.remove_class("hidden")

    def on_key(self, event: events.Key) -> None:
        """Handle key events."""
        if event.key == "escape":
            if self._expanded_id is not None:
                try:
                    output_panel = self.query_one("#commands-output-panel", Static)
                    output_panel.add_class("hidden")
                    output_panel.update("")
                    self._expanded_id = None
                except NoMatches:
                    pass
            else:
                self.dismiss(None)
            event.stop()
